@@ -8,11 +8,12 @@ from __future__ import unicode_literals
 
 from django.template.base import (TOKEN_BLOCK, TOKEN_COMMENT, TOKEN_TEXT,
                                   TOKEN_VAR, TRANSLATOR_COMMENT_MARK,
-                                  DebugLexer, Lexer, Parser)
+                                  DebugLexer, Parser)
 from django.template.defaulttags import token_kwargs
 from django.templatetags.i18n import do_block_translate, do_translate
 from django.utils.encoding import force_text
-from django.utils.translation import trim_whitespace
+from django.utils.html import escape as escape_html
+from transifex.common._compat import string_types, text_type
 from transifex.native.django.utils import templates
 from transifex.native.django.utils.templates import find_filter_identity
 from transifex.native.tools.migrations.models import (Confidence,
@@ -22,53 +23,21 @@ from transifex.native.tools.migrations.models import (Confidence,
 COMMENT_FOUND = object()
 
 
-def _render_native(str_format, content, params):
-    """Return a string with syntax compatible to Transifex Native template
-    tags.
+def _render_params(params):
+    """ Usage:
 
-    The given `params` will be serialized and will be added to the final
-    string, together with `content`, using specific placeholders
-    ('__tx_native_content__' and '__tx_native_params__').
+        >>> _render_params({'a': "b", 'c': "b"})
+        <<< 'a="b" c="d"'
 
-    Usage:
-    >>> _render_native(
-    >>>     '{% t __tx_native_content__ __tx_native_params__%}',
-    >>>     '"Call {name} at {phone_num}"',
-    >>>     {'username': 'john', 'phone_num': '(650) 111-1111'}
-    >>> )
-    <<< {% t "Call {name} at {phone_num}" username="john" phone_num="(650) 111-1111" %}
-
-    This function does not know anything else about how to structure the
-    template tag, so the caller needs to provide a suitable and valid
-    `str_format`.
-
-    :param unicode str_format: the structure of the template tag,
-        as described above
-    :param unicode content: the actual translatable content, that will
-        replace the `__tx_native_content__` placeholder inside `str_format`
-    :param dict params: a dictionary with parameters, that will
-        replace the `__tx_native_params__` placeholder inside `str_format`
-    :return: a new string that follows Transifex Native syntax for template
-        tags
-    :rtype: unicode
+        Handles some cornercases when values are None or the special value that
+        indicates a comment is expected later in the file.
     """
-    rendered_params = {}
-    for key, value in params.items():
-        if value and value != COMMENT_FOUND:
-            rendered_params[key] = value
 
-    serialized_params = ' '.join(
-        sorted([
-            '{key}={value}'.format(key=key, value=value)
-            for key, value in rendered_params.items()
-        ])
-    )
-    return str_format.replace(
-        '__tx_native_content__', content
-    ).replace(
-        '__tx_native_params__',
-        ('{} ' if serialized_params else '{}').format(serialized_params)
-    )
+    result = []
+    for key, value in sorted(params.items(), key=lambda i: i[0]):
+        if value and value != COMMENT_FOUND:
+            result.append('='.join((key, text_type(value))))
+    return ' '.join(result)
 
 
 def _make_plural(singular, plural, count_var):
@@ -167,12 +136,19 @@ def _retrieve_comment(token_contents):
     :rtype: unicode
     """
     mark = '{}:'.format(TRANSLATOR_COMMENT_MARK)
+    result = None
     if token_contents.startswith(mark):
-        return '"{}"'.format(token_contents.replace(mark, '').strip())
+        result = '"{}"'.format(token_contents.replace(mark, '').strip())
+    if result is not None and '\n' in result:
+        # Join multiline comments
+        result = ' '.join((line.strip()
+                           for line in result.splitlines()
+                           if line.strip()))
+    return result
 
 
 class DjangoTagMigrationBuilder(object):
-    """Parses Django templates and creates test_migrations for each template.
+    """Parses Django templates and creates file migrations for each template.
 
     A migration is an object that describes the changes that need to be made
     in order to change a template file that uses the Django i18n syntax
@@ -232,11 +208,11 @@ class DjangoTagMigrationBuilder(object):
         parser = Parser(tokens, libraries={}, builtins=[], origin=filename)
 
         # Since no template libraries are loaded when this code is running,
-        # we need to override the find function in order to use the functionality
-        # of the Parser class. The overridden function returns the object as given.
+        # we need to override the find function in order to use the
+        # functionality of the Parser class. The overridden function returns
+        # the object as given.
         # Without the override, a KeyError would be raised inside the parser.
         parser.find_filter = find_filter_identity
-
         # Create a migration object for this template; we'll add stuff to it
         # as we go
         migration = FileMigration(filename, src)
@@ -246,9 +222,10 @@ class DjangoTagMigrationBuilder(object):
 
             # Parse the current token. This may or may not return a migration.
             # Also it may return a list of tokens that were consumed,
-            # additionally to the current token. If this happens, `_parse_token()`
-            # will have made sure that `parser` has moved forward, consuming those
-            # tokens, so that they don't appear again in the while loop.
+            # additionally to the current token. If this happens,
+            # `_parse_token()` will have made sure that `parser` has moved
+            # forward, consuming those tokens, so that they don't appear again
+            # in the while loop.
             string_migration, extra_consumed_tokens = self._parse_token(
                 token, parser, original_string=src[start:end]
             )
@@ -277,8 +254,8 @@ class DjangoTagMigrationBuilder(object):
             specific line in the template, e.g. contains 'trans "A string"'
         :param django.template.base.Parser parser: the parser object that holds
             information on the whole template document
-        :param unicode original_string: the full string matched in the template,
-            e.g. '{% trans "A string" %}'
+        :param unicode original_string: the full string matched in the
+            template, e.g. '{% trans "A string" %}'
 
         :return: a StringMigration object that contains information about
             the Django and Transifex Native syntax of a specific set of strings
@@ -291,15 +268,31 @@ class DjangoTagMigrationBuilder(object):
 
         # A comment tag was found, so we need to retrieve the comment text
         # e.g. {# Translators: Do this and that #}
+        # If it's a generic (non-translator) comment, ignore
         elif token.token_type == TOKEN_COMMENT:
-            self._comment = _retrieve_comment(token.contents)
-            self._current_string_migration = StringMigration(
-                original_string, '')
-            return None, None
+            comment = _retrieve_comment(token.contents)
+
+            if comment:
+                self._comment = comment
+                self._current_string_migration = StringMigration(
+                    original_string, '')
+                return None, None
 
         # A variable was found; copy as is
         elif token.token_type == TOKEN_VAR:
-            return StringMigration(original_string, original_string), None
+            # that's a special case we need to take care of:
+            # {{ _("Are you sure you want to remove the ($(collaborator_count)) selected collaborators?")|escapejs }}
+            if token.contents.startswith('_('):
+                token.token_type = TOKEN_BLOCK
+                clos_par_pos = 0
+                for i, j in enumerate(token.contents):
+                    if j == ')':
+                        clos_par_pos = i
+                token.contents = ('trans ' +
+                                  token.contents[2:clos_par_pos] +
+                                  token.contents[clos_par_pos + 1:])
+            else:
+                return StringMigration(original_string, original_string), None
 
         return self._parse_block(token, parser, original_string)
 
@@ -331,7 +324,8 @@ class DjangoTagMigrationBuilder(object):
         if self._comment == COMMENT_FOUND:
             self._comment = _retrieve_comment(token.contents)
 
-            # Make sure to record that the tag is removed from the migrated result
+            # Make sure to record that the tag is removed from the migrated
+            # result
             if self._current_string_migration:
                 self._current_string_migration.update(original_string, '')
             return None, None
@@ -354,15 +348,17 @@ class DjangoTagMigrationBuilder(object):
         :param django.template.base.parser: the parser object
         :param unicode original_string: the string found in the template
         """
-        # Split the given token into its parts (includes the template tag name),
-        # e.g. "{% trans "This is the title" context "Some context" %}" returns:
-        # ['trans', '"This is the title"', 'context', '"Some context"']
+        # Split the given token into its parts (includes the template tag
+        # name), e.g. "{% trans "This is the title" context "Some context" %}"
+        # returns: ['trans', '"This is the title"', 'context', '"Some
+        # context"']
         bits = token.split_contents()
 
         tag_name = bits[0]
 
         # Right after {% load i18n %} append a {% load transifex %} tag
-        if tag_name == templates.LOAD_TAG and bits[1] == templates.DJANGO_i18n_TAG_NAME:
+        if (tag_name == templates.LOAD_TAG and
+                bits[1] == templates.DJANGO_i18n_TAG_NAME):
             # Make sure there is not already a tag that loads "transifex"
             # by checking all remaining nodes
             for t in parser.tokens:
@@ -370,7 +366,8 @@ class DjangoTagMigrationBuilder(object):
                     templates.LOAD_TAG in t.contents
                     and templates.TRANSIFEX_TAG_NAME in t.contents
                 ):
-                    return StringMigration(original_string, original_string), None
+                    return (StringMigration(original_string, original_string),
+                            None)
 
             string_migration = StringMigration(
                 original_string,
@@ -391,22 +388,30 @@ class DjangoTagMigrationBuilder(object):
         elif tag_name == templates.ENDWITH_TAG:
             self._with_kwargs.pop()
 
-        # A {% comment %} tag was found; expect the actual
-        # comment text to follow shortly
+        # A {% comment %} tag was found; If this is a translation comment,
+        # expect the actual comment text to follow shortly
         elif tag_name == templates.COMMENT_TAG:
-            self._comment = COMMENT_FOUND
-            # Create a string migration and start keeping track
-            # of all the strings that will be migrated
-            # within the following set of tokens that apply
-            self._current_string_migration = StringMigration(
-                original_string, '')
-            return None, None
-        # An {% endcomment %} tag was found; no need to do anything special,
-        # just make sure to record that the tag is removed from the migrated result
+            next_token = parser.tokens[0] if parser.tokens else None
+            if next_token.token_type == TOKEN_TEXT:
+                comment = _retrieve_comment(next_token.contents)
+                if comment:
+                    self._comment = COMMENT_FOUND
+                    # Create a string migration and start keeping track
+                    # of all the strings that will be migrated
+                    # within the following set of tokens that apply
+                    self._current_string_migration = StringMigration(
+                        original_string, '')
+                    return None, None
+
+        # An {% endcomment %} tag was found
         elif tag_name == templates.ENDCOMMENT_TAG:
-            if self._current_string_migration:
-                self._current_string_migration.update(original_string, '')
-            return None, None
+            # No need to do anything special, just make sure to record
+            # that the tag is removed from the migrated result
+            # If a translation comment wasn't open, ignore it
+            if self._comment is not None:
+                if self._current_string_migration:
+                    self._current_string_migration.update(original_string, '')
+                return None, None
 
         # A {% trans %} tag was found
         elif tag_name in templates.TRANSLATE_TAGS:
@@ -427,38 +432,45 @@ class DjangoTagMigrationBuilder(object):
         :param django.template.base.parser: the parser object
         :param unicode original_string: the string found in the template
         """
+
         # Use Django's do_translate() method to parse the token
         trans_node = do_translate(parser, token)
 
+        confidence = Confidence.LOW if trans_node.noop else Confidence.HIGH
+
         message_context = trans_node.message_context
-        text = trans_node.filter_expression.var.literal
-        # {% trans "Some text" %}
-        if text:
-            text = '"{}"'.format(text)
-        # {% trans some_var %}
+        # Our SDK supports filter expressions
+        text = trans_node.filter_expression.token
+
+        # Source strings that contain XML symbols should use 'ut'. We determine
+        # whether the string contains XML symbols by testing if an escaping
+        # attempt changes it in any way.
+        # eg `{% trans "a b" %}`            => `{% t "a b" %}`
+        #    `{% trans "<xml>a</xml> b" %}` => `{% ut "<xml>a</xml> b" %}`
+        if isinstance(trans_node.filter_expression.var, string_types):
+            literal = trans_node.filter_expression.var
         else:
-            # Our SDK currently does not support variables outside strings,
-            # but we can simulate the same behavior using a placeholder
-            text = '"{var}" var=var'.replace(
-                'var', trans_node.filter_expression.var.var
-            )
-
-        params = {
-            '_context': message_context,
-            '_comment': self._comment,
-        }
+            literal = trans_node.filter_expression.var.literal
+        if (isinstance(literal, string_types) and
+                escape_html(literal) != literal):
+            tag_name = "ut"
+        else:
+            tag_name = "t"
 
+        params = {'_context': message_context, '_comment': self._comment}
         # Reset the stored comment, so that it doesn't leak to the next token
         self._comment = None
 
-        return self._final_string_migration(
-            original_string,
-            _render_native(
-                '{% t __tx_native_content__ __tx_native_params__%}',
-                text,
-                params,
-            ),
-        )
+        # Render the final output
+        t_tag = ['{%', tag_name, text, _render_params(params)]
+        if trans_node.asvar:
+            t_tag.extend(['as', trans_node.asvar])
+        t_tag.append('%}')
+        t_tag = ' '.join((thing.strip() for thing in t_tag if thing.strip()))
+
+        return self._final_string_migration(original_string,
+                                            t_tag,
+                                            confidence=confidence)
 
     def _parse_blocktrans(self, token, parser, original_string):
         """Parse a {% blocktrans %} token and return a migration object.
@@ -467,6 +479,7 @@ class DjangoTagMigrationBuilder(object):
         :param django.template.base.parser: the parser object
         :param unicode original_string: the string found in the template
         """
+
         # Use Django's blocktranslate tag function to actually parse
         # the whole tag, so that we easily get all information
         # Internally, the do_block_translate() call will make the parser
@@ -484,16 +497,8 @@ class DjangoTagMigrationBuilder(object):
         singular_text = _render_var_tokens(blocktrans_node.singular)
         plural_text = _render_var_tokens(blocktrans_node.plural)
 
-        # Support the "trimmed" option of {% blocktrans %}
-        if blocktrans_node.trimmed:
-            singular_text = trim_whitespace(singular_text)
-            plural_text = trim_whitespace(plural_text)
-
         # Start building the parameters supported by Transifex Native
-        params = {
-            '_context': message_context,
-            '_comment': self._comment,
-        }
+        params = {'_context': message_context, '_comment': self._comment}
 
         # Plural support in Django works by using the "count" keyword
         counter_var = blocktrans_node.countervar
@@ -506,6 +511,7 @@ class DjangoTagMigrationBuilder(object):
             key: value.token
             for key, value in blocktrans_node.extra_context.items()
         })
+        params = _render_params(params)
 
         # Retrieve any variables inside text, e.g.
         # "This is a {{ var }} and this is {{ another_var }}"
@@ -513,33 +519,54 @@ class DjangoTagMigrationBuilder(object):
             _get_variable_names(blocktrans_node.singular) +
             _get_variable_names(blocktrans_node.plural)
         )
-        for variable in variables_in_text:
-            params.setdefault(variable, variable)
 
         # Reset the stored comment, so that it doesn't leak to the next token
         self._comment = None
 
         # Build the template of the tag for Transifex Native syntax
         is_multiline = '\n' in singular_text or plural_text
-        block_format = (
-            '{% t __tx_native_params__%}__tx_native_content__{% endt %}'
-            if is_multiline
-            else '{% t "__tx_native_content__" __tx_native_params__%}'
-        )
+        content = _make_plural(singular_text, plural_text, counter_var)
+
+        # Source strings that contain XML symbols should use 'ut'. We determine
+        # whether the string contains XML symbols by testing if an escaping
+        # attempt changes it in any way.
+        # eg `{% blocktrans %}a b{% endblocktrans %}` =>
+        #        `{% t "a b" %}`
+        # eg `{% blocktrans %}<xml>a</xml> b{% endblocktrans %}` =>
+        #        `{% ut "<xml>a</xml> b" %}`
+        if escape_html(content) != content:
+            tag_name = "ut"
+        else:
+            tag_name = "t"
+
+        # Render the final output
+        t_tag = ['{% ', tag_name]
+        if is_multiline or '"' in content:
+            if blocktrans_node.trimmed:
+                t_tag.append(' |trimmed')
+        else:
+            t_tag.extend([' "', content, '"'])
+            if blocktrans_node.trimmed:
+                t_tag.append('|trimmed')
+        if params.strip():
+            t_tag.extend([' ', params])
+        if blocktrans_node.asvar:
+            t_tag.extend([' as ', blocktrans_node.asvar])
+        t_tag.append(' %}')
+        if is_multiline:
+            t_tag.extend([content, '{% end', tag_name, ' %}'])
+        t_tag = ''.join(t_tag)
+
         # Determine the confidence of the migration
-        confidence = Confidence.HIGH if not variables_in_text else Confidence.LOW
+        confidence = (Confidence.HIGH
+                      if not variables_in_text
+                      else Confidence.LOW)
 
         # Create the actual migration
-        return self._final_string_migration(
-            original_string,
-            _render_native(
-                block_format,
-                _make_plural(singular_text, plural_text, counter_var),
-                params,
-            ),
-            consumed_tokens=consumed_tokens,
-            confidence=confidence
-        )
+        return self._final_string_migration(original_string,
+                                            t_tag,
+                                            consumed_tokens=consumed_tokens,
+                                            confidence=confidence)
 
     def _final_string_migration(
         self,
